"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getNodeDir = exports.getNodeVersionsBaseDir = exports.getNodeBinDir = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const fetch_1 = require("@pnpm/fetch");
const logger_1 = require("@pnpm/logger");
const node_fetcher_1 = require("@pnpm/node.fetcher");
const store_path_1 = __importDefault(require("@pnpm/store-path"));
const load_json_file_1 = __importDefault(require("load-json-file"));
const write_json_file_1 = __importDefault(require("write-json-file"));
const getNodeMirror_1 = __importDefault(require("./getNodeMirror"));
const parseNodeEditionSpecifier_1 = require("./parseNodeEditionSpecifier");
async function getNodeBinDir(opts) {
    const fetch = (0, fetch_1.createFetchFromRegistry)(opts);
    const nodesDir = getNodeVersionsBaseDir(opts.pnpmHomeDir);
    let wantedNodeVersion = opts.useNodeVersion ?? (await readNodeVersionsManifest(nodesDir))?.default;
    if (wantedNodeVersion == null) {
        const response = await fetch('https://registry.npmjs.org/node');
        wantedNodeVersion = (await response.json())['dist-tags'].lts; // eslint-disable-line
        if (wantedNodeVersion == null) {
            throw new Error('Could not resolve LTS version of Node.js');
        }
        await (0, write_json_file_1.default)(path_1.default.join(nodesDir, 'versions.json'), {
            default: wantedNodeVersion,
        });
    }
    const { versionSpecifier, releaseChannel } = (0, parseNodeEditionSpecifier_1.parseNodeEditionSpecifier)(wantedNodeVersion);
    const nodeMirrorBaseUrl = (0, getNodeMirror_1.default)(opts.rawConfig, releaseChannel);
    const nodeDir = await getNodeDir(fetch, {
        ...opts,
        useNodeVersion: versionSpecifier,
        nodeMirrorBaseUrl,
    });
    return process.platform === 'win32' ? nodeDir : path_1.default.join(nodeDir, 'bin');
}
exports.getNodeBinDir = getNodeBinDir;
function getNodeVersionsBaseDir(pnpmHomeDir) {
    return path_1.default.join(pnpmHomeDir, 'nodejs');
}
exports.getNodeVersionsBaseDir = getNodeVersionsBaseDir;
async function getNodeDir(fetch, opts) {
    const nodesDir = getNodeVersionsBaseDir(opts.pnpmHomeDir);
    await fs_1.default.promises.mkdir(nodesDir, { recursive: true });
    const versionDir = path_1.default.join(nodesDir, opts.useNodeVersion);
    if (!fs_1.default.existsSync(versionDir)) {
        const storeDir = await (0, store_path_1.default)({
            pkgRoot: process.cwd(),
            storePath: opts.storeDir,
            pnpmHomeDir: opts.pnpmHomeDir,
        });
        const cafsDir = path_1.default.join(storeDir, 'files');
        (0, logger_1.globalInfo)(`Fetching Node.js ${opts.useNodeVersion} ...`);
        await (0, node_fetcher_1.fetchNode)(fetch, opts.useNodeVersion, versionDir, {
            ...opts,
            cafsDir,
            retry: {
                maxTimeout: opts.fetchRetryMaxtimeout,
                minTimeout: opts.fetchRetryMintimeout,
                retries: opts.fetchRetries,
                factor: opts.fetchRetryFactor,
            },
        });
    }
    return versionDir;
}
exports.getNodeDir = getNodeDir;
async function readNodeVersionsManifest(nodesDir) {
    try {
        return await (0, load_json_file_1.default)(path_1.default.join(nodesDir, 'versions.json'));
    }
    catch (err) { // eslint-disable-line
        if (err.code === 'ENOENT') {
            return {};
        }
        throw err;
    }
}
//# sourceMappingURL=node.js.map